/****************************************************
 * This sample application for the PIC 16F84A drives
 * the LM032L LCD module.
 *
 * The application uses a 4-bit data bus
 * This application demonstrates how to use the EEPROM
 ****************************************************/

#include <htc.h>;

// define crystal speed
#define _XTAL_FREQ 4000000

// define names
#define RS	RB1		// RS word. 0=Instruction	, 1=Data
#define RW	RB2		// RW word. 0=write			, 1=read
#define CE 	RB3		// CE word. 0=disabled		, 1=enabled
#define busy RB7
#define MODE_DATA 1
#define MODE_CMD  0

// function prototypes
//void write (char data, char mode);
void writeFourbits(char data);
void writeData(char data);
void writeString(char *pMsg);
void writeCommand(char data);
void LCDbusy();
void LCDinitialize();
void main();

// fill the data
__EEPROM_DATA('H','e','l','l','o',' ','W','o');	// Hello World
__EEPROM_DATA('r','l','d','!',0,0,0,0);
__EEPROM_DATA('H','e','l','l','o',' ','M','o');	// Hello Moon
__EEPROM_DATA('o','n','!',0,0,0,0,0);
__EEPROM_DATA('b','v','s','y','s','t','e','m');	// bvsystems.be
__EEPROM_DATA('s','.','b','e',0,0,0,0);
__EEPROM_DATA('1','6','F','8','4','A',' ','D');	// 16F84A Demo
__EEPROM_DATA('e','m','o',0,0,0,0,0);



// Initialize the LCD module
// Set to 4-bit operation, no cursor
void LCDinitialize() {
	writeCommand(0x28);	// busy flag & address
	writeCommand(0x28);	// busy flag & address
	writeCommand(0x28);	// busy flag & address
	writeCommand(0x06);	// cursor move direction
	writeCommand(0x0C);	// LCD enable, cursor off
	writeCommand(0x01);	// clear LCD
}



// LCDbusy() will wait until the LCD isn't busy
void LCDbusy() {
	TRISB7=1;	// put port into input mode
	RS=0;
	RW=1;
	CE=1;
	while (busy==1);
	CE=0;
	TRISB7=0;	// put port into output mode
}


// writeFourBits writes a sequence of 4 bits to the LCD
void writeFourbits(char data) {
	PORTB=(PORTB&0x0F)|data;
	RW=0;
	__delay_us(1);
	CE=1;
	__delay_us(2);
	CE=0;
	__delay_us(3);	
}



// writeCommand writes a character to the LCD
void writeData(char data) {
	RS=MODE_DATA;
	writeFourbits(data&0xF0);
	writeFourbits((data&0x0F)<<4);
	LCDbusy();
}



// writeCommand writes a command to the LCD
void writeCommand(char data) {
	RS=MODE_CMD;
	writeFourbits(data&0xF0);
	writeFourbits((data&0x0F)<<4);
	LCDbusy();
}



// writeString() will write a String to the LCD
void writeString(char *pMsg) {
	writeCommand(0x01);	// clear LCD

    while(*pMsg!='\0')
		writeData(*pMsg++);
}



// showMessageFromEEPROM() will show a message from the EEPROM
void showMessageFromEEPROM(char msgAdr) {
	writeCommand(0x01);	// clear LCD
	char currentData;

	while((currentData=EEPROM_READ(msgAdr++))!='\0') 
		writeData(currentData);

	__delay_ms(1000);
}



// Program Main Loop
void main () {
	// define all ports as outputs
	TRISA=0x00;
	TRISB=0x00;
	PORTA=0x00;
	PORTB=0x00;

	// initialize the LCD module
	LCDinitialize();

	// run an endless loop
	while (1) {
		if (RA0==1) showMessageFromEEPROM(0x00);
		if (RA1==1) showMessageFromEEPROM(0x10);
		if (RA2==1) showMessageFromEEPROM(0x20);
		if (RA3==1) showMessageFromEEPROM(0x30);
	};
}


